@@ -13,8 +13,8 @@ from Orange.widgets.utils.sql import check_sql_input
 
 
 class OWKMeans(widget.OWWidget):
-    name = "k-means"
-    description = "k-means clustering algorithm with silhouette-based " \
+    name = "k-Means"
+    description = "k-Means clustering algorithm with silhouette-based " \
                   "quality estimation."
     icon = "icons/KMeans.svg"
     priority = 2100
@@ -39,7 +39,7 @@ class OWKMeans(widget.OWWidget):
 
     resizing_enabled = False
 
-    k = Setting(8)
+    k = Setting(3)
     k_from = Setting(2)
     k_to = Setting(8)
     optimize_k = Setting(False)
@@ -50,7 +50,7 @@ class OWKMeans(widget.OWWidget):
     append_cluster_ids = Setting(True)
     place_cluster_ids = Setting(OUTPUT_CLASS)
     output_name = Setting("Cluster")
-    auto_run = Setting(False)
+    auto_run = Setting(True)
 
     def __init__(self):
         super().__init__()
@@ -61,7 +61,7 @@ class OWKMeans(widget.OWWidget):
 
         box = gui.vBox(self.controlArea, "Number of Clusters")
         layout = QGridLayout()
-        bg = gui.radioButtonsInBox(
+        self.n_clusters = bg = gui.radioButtonsInBox(
             box, self, "optimize_k", [], orientation=layout,
             callback=self.update)
         layout.addWidget(
@@ -132,7 +132,10 @@ class OWKMeans(widget.OWWidget):
                      callback=self.send_data)
 
         gui.separator(self.buttonsArea, 30)
-        gui.auto_commit(self.buttonsArea, self, "auto_run", "Apply", box=None)
+        self.apply_button = gui.auto_commit(
+            self.buttonsArea, self, "auto_run", "Apply", box=None,
+            commit=self.commit
+        )
         gui.rubber(self.controlArea)
 
         self.table_model = QStandardItemModel(self)
@@ -199,12 +202,16 @@ class OWKMeans(widget.OWWidget):
         self.updateOptimizationGui()
         self.update()
 
-    def check_data_size(self, n, msg_func):
+    def check_data_size(self, n, msg_group):
+        msg_group.add_message(
+            "not_enough_data",
+            "Too few ({}) unique data instances for {} clusters")
         if n > len(self.data):
-            msg_func("Too few unique data instances ({}) for {} clusters".
-                     format(len(self.data), n))
+            msg_group.not_enough_data(len(self.data), n)
             return False
-        return True
+        else:
+            msg_group.not_enough_data.clear()
+            return True
 
     def run_optimization(self):
         # Disabling is needed since this function is not reentrant
@@ -212,13 +219,14 @@ class OWKMeans(widget.OWWidget):
         try:
             self.controlArea.setDisabled(True)
             self.optimization_runs = []
-            if not self.check_data_size(self.k_from, self.error):
+            if not self.check_data_size(self.k_from, self.Error):
                 return
-            self.check_data_size(self.k_to, self.warning)
+            self.check_data_size(self.k_to, self.Warning)
             k_to = min(self.k_to, len(self.data))
             kmeans = KMeans(
                 init=['random', 'k-means++'][self.smart_init],
-                n_init=self.n_init, max_iter=self.max_iterations)
+                n_init=self.n_init, max_iter=self.max_iterations,
+                compute_silhouette_score=self.scoring == self.SILHOUETTE)
             with self.progressBar(k_to - self.k_from + 1) as progress:
                 for k in range(self.k_from, k_to + 1):
                     progress.advance()
@@ -230,7 +238,7 @@ class OWKMeans(widget.OWWidget):
         self.send_data()
 
     def cluster(self):
-        if not self.check_data_size(self.k, self.error):
+        if not self.check_data_size(self.k, self.Error):
             return
         self.km = KMeans(
             n_clusters=self.k,
@@ -240,8 +248,7 @@ class OWKMeans(widget.OWWidget):
         self.send_data()
 
     def run(self):
-        self.error()
-        self.warning()
+        self.clear_messages()
         if not self.data:
             return
         if self.optimize_k:
@@ -249,7 +256,8 @@ class OWKMeans(widget.OWWidget):
         else:
             self.cluster()
 
-    commit = run
+    def commit(self):
+        self.run()
 
     def show_results(self):
         minimize = self.SCORING_METHODS[self.scoring][2]
@@ -264,8 +272,8 @@ class OWKMeans(widget.OWWidget):
         best_run = scores.index(best_score)
         score_span = (best_score - worst_score) or 1
         max_score = max(scores)
-        nplaces = min(5, int(abs(math.log(max(max_score, 1e-10)))) + 2)
-        fmt = "{{:.{}f}}".format(nplaces)
+        nplaces = min(5, np.floor(abs(math.log(max(max_score, 1e-10)))) + 2)
+        fmt = "{{:.{}f}}".format(int(nplaces))
         model = self.table_model
         model.setRowCount(len(k_scores))
         for i, (k, score) in enumerate(k_scores):
@@ -278,7 +286,8 @@ class OWKMeans(widget.OWWidget):
             item = model.item(i, 1)
             if item is None:
                 item = QStandardItem()
-            item.setData(fmt.format(score), Qt.DisplayRole)
+            item.setData(fmt.format(score) if not np.isnan(score) else 'out-of-memory error',
+                         Qt.DisplayRole)
             bar_ratio = 0.95 * (score - worst_score) / score_span
             item.setData(bar_ratio, gui.TableBarItem.BarRole)
             model.setItem(i, 1, item)
@@ -348,15 +357,18 @@ class OWKMeans(widget.OWWidget):
         self.data = data
         if data is None:
             self.table_model.setRowCount(0)
+            self.send("Annotated Data", None)
+            self.send("Centroids", None)
         else:
             self.data = data
             self.run()
 
     def send_report(self):
+        k_clusters = self.k
+        if self.optimize_k and self.optimization_runs and self.selected_row() is not None:
+            k_clusters = self.optimization_runs[self.selected_row()][1].k
         self.report_items((
-            ("Number of clusters",
-             self.optimization_runs[self.selected_row()][1].k
-             if self.optimize_k else self.k),
+            ("Number of clusters", k_clusters),
             ("Optimization",
              self.optimize_k != 0 and
              "{}, {} re-runs limited to {} steps".format(
