@@ -149,7 +149,10 @@ class PyTableModel(QAbstractTableModel):
                                           1 if strlen < 5 else
                                           0 if strlen < 6 else
                                           3,
-                                          'f' if strlen < 6 and absval >= .001 else 'e')
+                                          'f' if (absval == 0 or
+                                                  absval >= .001 and
+                                                  strlen < 6)
+                                          else 'e')
             return str(value)
         if role == Qt.TextAlignmentRole and isinstance(value, Number):
             return Qt.AlignRight | Qt.AlignVCenter
@@ -583,12 +586,13 @@ class PyListModelTooltip(PyListModel):
 
 
 class VariableListModel(PyListModel):
-
     MIME_TYPE = "application/x-Orange-VariableList"
 
     def data(self, index, role=Qt.DisplayRole):
         if self._is_index_valid_for(index, self):
             var = self[index.row()]
+            if var is None and role == Qt.DisplayRole:
+                return "None"
             if not isinstance(var, Variable):
                 return super().data(index, role)
             elif role == Qt.DisplayRole:
@@ -597,6 +601,8 @@ class VariableListModel(PyListModel):
                 return gui.attributeIconDict[var]
             elif role == Qt.ToolTipRole:
                 return self.variable_tooltip(var)
+            elif role == gui.TableVariable:
+                return var
             else:
                 return PyListModel.data(self, index, role)
 
