@@ -453,11 +453,11 @@ _define_symbols()
 
 
 class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
-    attr_color = ContextSetting("", ContextSetting.OPTIONAL)
-    attr_label = ContextSetting("", ContextSetting.OPTIONAL)
+    attr_color = ContextSetting("", ContextSetting.OPTIONAL, exclude_metas=False)
+    attr_label = ContextSetting("", ContextSetting.OPTIONAL, exclude_metas=False)
     label_only_selected = Setting(False)
-    attr_shape = ContextSetting("", ContextSetting.OPTIONAL)
-    attr_size = ContextSetting("", ContextSetting.OPTIONAL)
+    attr_shape = ContextSetting("", ContextSetting.OPTIONAL, exclude_metas=False)
+    attr_size = ContextSetting("", ContextSetting.OPTIONAL, exclude_metas=False)
 
     point_width = Setting(10)
     alpha_value = Setting(128)
@@ -472,8 +472,6 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
     DarkerValue = 120
     UnknownColor = (168, 50, 168)
 
-    ID_MISSING_COORDS, ID_MISSING_SIZE, ID_MISSING_SHAPE = range(1, 4)
-
     def __init__(self, scatter_widget, parent=None, _="None"):
         gui.OWComponent.__init__(self, scatter_widget)
         self.view_box = InteractiveViewBox(self)
@@ -492,6 +490,19 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.labels = []
 
         self.master = scatter_widget
+        self.master.Warning.add_message(
+            "missing_coords",
+            "Plot cannot be displayed because '{}' or '{}' is missing for "
+            "all data points")
+        self.master.Information.add_message(
+            "missing_coords",
+            "Points with missing '{}' or '{}' are not displayed")
+        self.master.Information.add_message(
+            "missing_size",
+            "Points with undefined '{}' are shown in smaller size")
+        self.master.Information.add_message(
+            "missing_shape",
+            "Points with undefined '{}' are shown as crossed circles")
         self.shown_attribute_indices = []
         self.shown_x = ""
         self.shown_y = ""
@@ -527,6 +538,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
     def new_data(self, data, subset_data=None, **args):
         self.plot_widget.clear()
+        self.remove_legend()
 
         self.density_img = None
         self.scatterplot_item = None
@@ -557,8 +569,8 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         self.set_axis_title("left", "")
 
     def update_data(self, attr_x, attr_y, reset_view=True):
-        self.master.warning(self.ID_MISSING_COORDS)
-        self.master.information(self.ID_MISSING_COORDS)
+        self.master.Warning.missing_coords.clear()
+        self.master.Information.missing_coords.clear()
         self._clear_plot_widget()
 
         self.shown_x = attr_x
@@ -567,8 +579,8 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         if self.scaled_data is None or not len(self.scaled_data):
             self.valid_data = None
         else:
-            index_x = self.attribute_name_index[attr_x]
-            index_y = self.attribute_name_index[attr_y]
+            index_x = self.data_domain.index(attr_x)
+            index_y = self.data_domain.index(attr_y)
             self.valid_data = self.get_valid_list([index_x, index_y],
                                                   also_class_if_exists=False)
             if not np.any(self.valid_data):
@@ -576,10 +588,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         if self.valid_data is None:
             self.selection = None
             self.n_points = 0
-            self.master.warning(
-                self.ID_MISSING_COORDS,
-                "Plot cannot be displayed because '{}' or '{}' is missing for "
-                "all data points".format(self.shown_x, self.shown_y))
+            self.master.Warning.missing_coords(self.shown_x, self.shown_y)
             return
 
         x_data, y_data = self.get_xy_data_positions(
@@ -619,10 +628,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
 
         data_indices = np.flatnonzero(self.valid_data)
         if len(data_indices) != self.original_data.shape[1]:
-            self.master.information(
-                self.ID_MISSING_COORDS,
-                "Points with missing '{}' or '{}' are not displayed".
-                format(self.shown_x, self.shown_y))
+            self.master.Information.missing_coords(self.shown_x, self.shown_y)
 
         self.scatterplot_item = ScatterPlotItem(
             x=x_data, y=y_data, data=data_indices,
@@ -676,11 +682,11 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         size_index = -1
         attr_size = self.attr_size
         if attr_size != "" and attr_size != "(Same size)":
-            size_index = self.attribute_name_index[attr_size]
+            size_index = self.data_domain.index(attr_size)
         return size_index
 
     def compute_sizes(self):
-        self.master.information(self.ID_MISSING_SIZE)
+        self.master.Information.missing_size.clear()
         size_index = self.get_size_index()
         if size_index == -1:
             size_data = np.full((self.n_points,), self.point_width)
@@ -692,10 +698,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         nans = np.isnan(size_data)
         if np.any(nans):
             size_data[nans] = self.MinShapeSize - 2
-            self.master.information(
-                self.ID_MISSING_SIZE,
-                "Points with undefined '{}' are shown in smaller size".
-                format(self.attr_size))
+            self.master.Information.missing_size(self.attr_size)
         return size_data
 
     def update_sizes(self):
@@ -710,7 +713,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         color_index = -1
         attr_color = self.attr_color
         if attr_color != "" and attr_color != "(Same color)":
-            color_index = self.attribute_name_index[attr_color]
+            color_index = self.data_domain.index(attr_color)
             color_var = self.data_domain[attr_color]
             colors = color_var.colors
             if color_var.is_discrete:
@@ -758,7 +761,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             pen = [make_pen(color, 1.5)] * self.n_points
             if subset is not None:
                 brush = [(QBrush(QColor(128, 128, 128, 0)),
-                          QBrush(QColor(128, 128, 128, self.alpha_value)))[s]
+                          QBrush(QColor(128, 128, 128, 255)))[s]
                          for s in subset]
             else:
                 brush = [QBrush(QColor(128, 128, 128, self.alpha_value))] \
@@ -784,7 +787,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                                    for col in self.pen_colors.tolist()]
             if subset is not None:
                 self.brush_colors[:, 3] = 0
-                self.brush_colors[subset, 3] = self.alpha_value
+                self.brush_colors[subset, 3] = 255
             else:
                 self.brush_colors[:, 3] = self.alpha_value
             pen = self.pen_colors
@@ -803,9 +806,10 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
                     [make_pen(QColor(*col).darker(self.DarkerValue), 1.5)
                      for col in colors])
                 self.pen_colors = pens[c_data]
+                alpha = self.alpha_value if subset is None else 255
                 self.brush_colors = np.array([
                     [QBrush(QColor(0, 0, 0, 0)),
-                     QBrush(QColor(col[0], col[1], col[2], self.alpha_value))]
+                     QBrush(QColor(col[0], col[1], col[2], alpha))]
                     for col in colors])
                 self.brush_colors = self.brush_colors[c_data]
             if subset is not None:
@@ -868,11 +872,11 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
         if attr_shape and attr_shape != "(Same shape)" and \
                 len(self.data_domain[attr_shape].values) <= \
                 len(self.CurveSymbols):
-            shape_index = self.attribute_name_index[attr_shape]
+            shape_index = self.data_domain.index(attr_shape)
         return shape_index
 
     def compute_symbols(self):
-        self.master.information(self.ID_MISSING_SHAPE)
+        self.master.Information.missing_shape.clear()
         shape_index = self.get_shape_index()
         if shape_index == -1:
             shape_data = self.CurveSymbols[np.zeros(self.n_points, dtype=int)]
@@ -881,10 +885,7 @@ class OWScatterPlotGraph(gui.OWComponent, ScaleScatterPlotData):
             nans = np.isnan(shape_data)
             if np.any(nans):
                 shape_data[nans] = len(self.CurveSymbols) - 1
-                self.master.information(
-                    self.ID_MISSING_SHAPE,
-                    "Points with undefined '{}' are shown as crossed circles".
-                    format(self.attr_shape))
+                self.master.Information.missing_shape(self.attr_shape)
             shape_data = self.CurveSymbols[shape_data.astype(int)]
         return shape_data
 
