@@ -52,13 +52,11 @@ class SieveRank(VizRankDialogAttrPair):
 
 
 class OWSieveDiagram(OWWidget):
-    """
-    A two-way contingency table providing information on the relation
-    between the observed and expected frequencies of a combination of values
-    """
     name = "Sieve Diagram"
+    description = "Visualize the observed and expected frequencies " \
+                  "for a combination of values."
     icon = "icons/SieveDiagram.svg"
-    priority = 310
+    priority = 200
 
     inputs = [("Data", Table, "set_data", Default),
               ("Features", AttributeList, "set_input_features")]
@@ -69,8 +67,8 @@ class OWSieveDiagram(OWWidget):
     want_control_area = False
 
     settingsHandler = DomainContextHandler()
-    attrX = ContextSetting("")
-    attrY = ContextSetting("")
+    attrX = ContextSetting("", exclude_metas=False)
+    attrY = ContextSetting("", exclude_metas=False)
     selection = ContextSetting(set())
 
     def __init__(self):
@@ -94,11 +92,9 @@ class OWSieveDiagram(OWWidget):
         self.attrXCombo = gui.comboBox(value="attrX", **combo_args)
         gui.widgetLabel(self.attr_box, "\u2715", sizePolicy=fixed_size)
         self.attrYCombo = gui.comboBox(value="attrY", **combo_args)
-        self.vizrank = SieveRank(self)
-        self.vizrank_button = gui.button(
-            self.attr_box, self, "Score Combinations", sizePolicy=fixed_size,
-            callback=self.vizrank.reshow, enabled=False)
-        self.vizrank.pairSelected.connect(self.set_attr)
+        self.vizrank, self.vizrank_button = SieveRank.add_vizrank(
+            self.attr_box, self, "Score Combinations", self.set_attr)
+        self.vizrank_button.setSizePolicy(*fixed_size)
 
         self.canvas = QGraphicsScene()
         self.canvasView = ViewWithPress(
@@ -122,14 +118,6 @@ class OWSieveDiagram(OWWidget):
         super().showEvent(event)
         self.update_graph()
 
-    def closeEvent(self, event):
-        self.vizrank.close()
-        super().closeEvent(event)
-
-    def hideEvent(self, event):
-        self.vizrank.hide()
-        super().hideEvent(event)
-
     def set_data(self, data):
         """
         Discretize continuous attributes, and put all attributes and discrete
