@@ -88,8 +88,6 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
     #: Short widget description (:class:`str` optional), displayed in
     #: canvas help tooltips.
     description = None
-    #: A longer widget description (:class:`str` optional)
-    long_description = None
     #: Widget icon path relative to the defining module
     icon = "icons/Unknown.png"
     #: Widget priority used for sorting within a category
@@ -159,7 +157,7 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
     #: :type: list of :class:`Message`
     UserAdviceMessages = []
 
-    def __new__(cls, *args, **kwargs):
+    def __new__(cls, *args, captionTitle=None, **kwargs):
         self = super().__new__(cls, None, cls.get_flags())
         QDialog.__init__(self, None, self.get_flags())
         WidgetMessagesMixin.__init__(self)
@@ -178,8 +176,11 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
         OWWidget.widget_id += 1
         self.widget_id = OWWidget.widget_id
 
-        if self.name:
-            self.setCaption(self.name)
+        captionTitle = self.name if captionTitle is None else captionTitle
+
+        # must be set without invoking setCaption
+        self.captionTitle = captionTitle
+        self.setWindowTitle(captionTitle)
 
         self.setFocusPolicy(Qt.StrongFocus)
 
@@ -204,6 +205,9 @@ class OWWidget(QDialog, Report, ProgressBarMixin, WidgetMessagesMixin,
 
         sc = QShortcut(QKeySequence.Copy, self)
         sc.activated.connect(self.copy_to_clipboard)
+        if self.controlArea is not None:
+            # Otherwise, the first control has focus
+            self.controlArea.setFocus(Qt.ActiveWindowFocusReason)
         return self
 
     # pylint: disable=super-init-not-called
@@ -767,6 +771,8 @@ Single = widget_description.Single
 #: Signal handlers with this flag have (object, id: object) -> None signature.
 Multiple = widget_description.Multiple
 #: Applies to user interaction only.
+#: Only connected if specifically requested (in a dedicated "Links" dialog)
+#: or it is the only possible connection.
 Explicit = widget_description.Explicit
 #: Dynamic output type.
 #: Specifies that the instances on the output will in general be
